package com.seitenbau.testing.addons.dbunit;

import java.io.InputStream;

import org.dbunit.AbstractDatabaseTester;
import org.dbunit.database.QueryDataSet;
import org.dbunit.dataset.AbstractDataSet;
import org.dbunit.dataset.DataSetException;
import org.dbunit.dataset.DefaultDataSet;
import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.ITable;
import org.dbunit.dataset.ITableIterator;
import org.dbunit.dataset.filter.DefaultColumnFilter;
import org.dbunit.dataset.xml.XmlDataSet;

import com.seitenbau.junit.ITestContext;
import com.seitenbau.junit.model.DDTModelTest;
import com.seitenbau.testing.addons.dbunit.listener.DbUnitListenerConfiguration;
import com.seitenbau.testing.addons.dbunit.listener.DbUnitTableConfig;
import com.seitenbau.testing.addons.plugin.DbUnitXStreamSweetner;
import com.seitenbau.testing.shared.locator.IProvidesTestPathClassLocator;
import com.seitenbau.testing.shared.locator.ITestPathClassBasedLocator;
import com.thoughtworks.xstream.XStream;

/**
 * Hilfsklasse zum Bereitstellen von Funktionen fr DBUnit Tests.
 *
 * <p>Implementiert eine automatische Filterung der Tabellen nach der Vorgabe
 * aus der XML Konfigurationsdatei.
 */
public class DBUnitTestHelper
{
  private AbstractDatabaseTester myDatabaseTester;

  private AbstractDataSet myPrestateDataSet;

  private Integer testID;

  private ITestContext myContext;

  private DbUnitListenerConfiguration myConfiguration;

  private ITestPathClassBasedLocator locator;

  private IConnectionProvider myConnectionProvider;

  public DBUnitTestHelper(ITestContext context, String testClassName)
      throws Exception
  {
    init(context, context.getTestItem(), testClassName);

  }

  public void init(ITestContext context, DDTModelTest testItem,
      String testClassName) throws Exception
  {
    myContext = context;
    testID = testItem.getId();
    // get configuration
    // String file = myFilePrefix + "Test.dbunit.config.xml";

    locator = ((IProvidesTestPathClassLocator) myContext).getLocator();
    String filename = locator.getTestResourceFile(
        DBUnitGeneratorConstants.PLUGIN_ID, DBUnitGeneratorConstants.CONFIG_FILE_NAME
            + ".xml");
    InputStream cfgStream = context.getTestClass()
        .getResourceAsStream(filename);
    if (cfgStream == null)
    {
      throw new IllegalArgumentException("Unable to load file : " + filename);
    }
    XStream xs = new XStream();
    new DbUnitXStreamSweetner().registerOnXStream(xs);
    myConfiguration = (DbUnitListenerConfiguration) xs.fromXML(cfgStream);
    IConnectionProviderConfig pconfig = myConfiguration.getConnectionProvider();
    myConnectionProvider = pconfig.getConnectionProviderClass()
        .newInstance();
    myConnectionProvider.setConfiguration(myConfiguration.getConnectionProvider());
    myDatabaseTester = myConnectionProvider.getDatabaseTester();

    String fileprestate = locator.getTestResourceFile(
        DBUnitGeneratorConstants.PLUGIN_ID, testID,
        DBUnitGeneratorConstants.PRESTATE_FILE_NAME + ".xml");
    InputStream stream = context.getTestClass().getResourceAsStream(
        fileprestate);

    myPrestateDataSet=new XmlDataSet(stream);
//    myPrestateDataSet = new FlatXmlDataSet(stream);
  }

  public AbstractDataSet getPrestateDataSet()
  {
    return myPrestateDataSet;
  }

  public AbstractDatabaseTester getDatabaseTester()
  {
    return myDatabaseTester;
  }

  public void setDatabaseTester(AbstractDatabaseTester tester)
  {
    myDatabaseTester = tester;
  }

  public IDataSet createActualDataSet() throws Exception
  {

    IDataSet databaseDataSet = null;
    if (myConfiguration.getTables() != null
        && myConfiguration.getTables().size() > 0)
    {
      myDatabaseTester = myConnectionProvider.getDatabaseTester();
      QueryDataSet partialDataSet = new QueryDataSet(myDatabaseTester
          .getConnection());
      for (DbUnitTableConfig table : myConfiguration.getTables())
      {
        String name = table.getTableNname();
        partialDataSet.addTable(name);
      }
      IDataSet ds = filterColumnsOut(partialDataSet);

      databaseDataSet = ds;

    }
    else
    {
      databaseDataSet =      filterColumnsOut(myDatabaseTester.getConnection().createDataSet());

    }

    return databaseDataSet;
  }

  private IDataSet filterColumnsOut(IDataSet partialDataSet)
      throws DataSetException
  {
    DefaultDataSet ds = new DefaultDataSet();

    ITableIterator tables = partialDataSet.iterator();

    while (tables.next())
    {
      ITable tab = tables.getTable();

      String[] ecolum=excludeColumnsList(tab);
      if(ecolum != null) {
        tab = DefaultColumnFilter.excludedColumnsTable(tab, ecolum );
      }
      ds.addTable( tab );
    }
    return ds;
  }

  private String[] excludeColumnsList(ITable tab)
  {
    DbUnitTableConfig tconfig=getTableConfigFor(tab);
    if( tconfig!=null) {
      String[] arObj = tconfig.getExcludeColumns().toArray( new String[]{});
      return arObj;
    }
    return null;
  }

  private DbUnitTableConfig getTableConfigFor(ITable tab)
  {
    String thename = tab.getTableMetaData().getTableName();
    for(DbUnitTableConfig tConfig:myConfiguration.getTables()) {
      String name = tConfig.getTableNname();
      if(name.equals( thename ) ) {
        return tConfig;
      }
    }
    return null;
  }

  public IDataSet createExpectedDataSet() throws Exception
  {
    String fileprestate = locator.getTestResourceFile(
        DBUnitGeneratorConstants.PLUGIN_ID, testID,
        DBUnitGeneratorConstants.POSTSTATE_FILE_NAME + ".xml");
    InputStream stream = myContext.getTestClass().getResourceAsStream(
        fileprestate);

    IDataSet expectedDataSet = new XmlDataSet(stream);
//    IDataSet expectedDataSet = new FlatXmlDataSet(stream);

    IDataSet ds = filterColumnsOut(expectedDataSet);

    // ITable expectedTable =
    // DefaultColumnFilter.excludedColumnsTable(
    // expectedDataSet.getTable("bestellung"),
    // new String[]{"id"}
    // );

    return ds;
  }

  public DbUnitListenerConfiguration getConfiguration()
  {
    return myConfiguration;
  }

  public void setConfiguration(DbUnitListenerConfiguration configuration)
  {
    myConfiguration = configuration;
  }

}
