package com.seitenbau.testing.addons.spring.dbunit;

import javax.persistence.EntityManager;
import javax.sql.DataSource;

import org.aspectj.lang.JoinPoint;
import org.dbunit.AbstractDatabaseTester;
import org.dbunit.database.IDatabaseConnection;
import org.springframework.web.context.ContextLoader;
import org.springframework.web.context.WebApplicationContext;

import com.seitenbau.testing.addons.dbunit.IConnectionProvider;
import com.seitenbau.testing.addons.dbunit.IConnectionProviderConfig;
import com.seitenbau.testing.addons.dbunit.helper.NotCloseingDatabaseConnection;
import com.seitenbau.testing.addons.dbunit.helper.NotCloseingDatabaseTester;
import com.seitenbau.testing.shared.helper.ReflectionHelper;
import com.seitenbau.testing.shared.tracer.IAspectJAware;

/**
 * Connection Provider fr Spring.
 *
 * <p>
 * Enthlt Code zum automatischen erkennen eines Entity Managers um
 * diesen per flush mit der aktuellen Transaktion zu synchronisieren.
 *
 * <p>
 * Sollte die SpringConnectionDataSourceInjectorBean nicht in Spring
 * konfiguriert sein, dann wird versuch ber den WebApplicationContext
 * die Datasource zu erhalten.
 */
public class ConnectionProviderSpringProvider implements IConnectionProvider,
    IAspectJAware
{
  private ConnectionProviderSpringConfig mySpringConnectionProviderConfig;

  private String dsName = "dataSource";

  public IDatabaseConnection getConnection()
  {
    DataSource ds = getDataSource();
    if (ds != null)
    {
      return new NotCloseingDatabaseConnection(ds);
    }
    else
    {
      return null;
    }
  }

  private DataSource getDataSource()
  {
    DataSource ds = SpringConnectionDataSourceInjectorBean
        .getCurrentDataSource();
    if (ds != null)
    {
      return ds;
    }

    WebApplicationContext wsctx = ContextLoader
        .getCurrentWebApplicationContext();
    if (wsctx != null)
    {
      if (dsName == null || dsName.length() < 1)
      {
        dsName = "dataSource";
      }
      Object bean = wsctx.getBean(dsName);
      if (bean instanceof DataSource)
      {
        return (DataSource) bean;
      }
    }
    return null;
  }

  public void setConfiguration(IConnectionProviderConfig connectionProvider)
  {
    if (connectionProvider instanceof ConnectionProviderSpringConfig)
    {
      mySpringConnectionProviderConfig = (ConnectionProviderSpringConfig) connectionProvider;
      dsName = mySpringConnectionProviderConfig.getDataSourceName();
    }
  }

  public AbstractDatabaseTester getDatabaseTester()
  {
    return new NotCloseingDatabaseTester(SpringConnectionDataSourceInjectorBean
        .getCurrentDataSource());
  }

  private void detectAndCallFlush(JoinPoint thisJoinPoint)
  {
    if (!getConfiguration().isForceFlush())
    {
      return;
    }
    EntityManager entityManger = detectEntityManger(thisJoinPoint);
    if (entityManger != null)
    {
      // EntityManager::flush synchronizes the Objects inside the
      // EntitiyManager with
      // the current Transaction. This is Transaction aware. So if the
      // Transaction is
      // rolled back, still nothing will be written into the Database.
      entityManger.flush();
    }
  }

  private ConnectionProviderSpringConfig getConfiguration()
  {
    if (mySpringConnectionProviderConfig == null)
    {
      return new ConnectionProviderSpringConfig();
    }
    return mySpringConnectionProviderConfig;
  }

  private EntityManager detectEntityManger(JoinPoint thisJoinPoint)
  {
    Object targetInstance = thisJoinPoint.getTarget();
    EntityManager em = (EntityManager) ReflectionHelper
        .getFieldValueContaining(targetInstance, EntityManager.class);
    return em;
  }

  public void enteringTarget(JoinPoint thisJoinPoint)
  {
    detectAndCallFlush(thisJoinPoint);
  }

  public void exceptionThrownTarget(JoinPoint thisJoinPoint,
      Throwable theThrowable)
  {
    detectAndCallFlush(thisJoinPoint);
  }

  public void proceededTarget(JoinPoint thisJoinPoint)
  {
    detectAndCallFlush(thisJoinPoint);
  }

}
